@@ -54,8 +54,6 @@ class Stream(object):
     为了和tornado的IOStream接口匹配
     """
 
-    _close_callback = None
-
     def __init__(self, sock, max_buffer_size=None,
                  read_chunk_size=4096):
         self.sock = sock
@@ -68,10 +66,7 @@ class Stream(object):
         self._read_regex = None
         self._read_bytes = None
         self._read_until_close = False
-        self._read_callback = None
-        self._streaming_callback = None
         self._read_checker = None
-        self._close_callback = None
 
         self.reader_count = 0
         self.writer_count = 0
@@ -99,90 +94,54 @@ class Stream(object):
             return
         safe_call(self.sock.shutdown, how)
 
-    def set_close_callback(self, callback):
-        """
-        设置回调
-        """
-        self._close_callback = callback
-
     @count_reader
-    def read_until_regex(self, regex, callback):
-        """Run ``callback`` when we read the given regex pattern.
-
-        The callback will get the data read (including the data that
-        matched the regex and anything that came before it) as an argument.
+    def read_until_regex(self, regex):
+        """Run when we read the given regex pattern.
         """
-        self._set_read_callback(callback)
         self._read_regex = re.compile(regex)
         while True:
-            # 0 代表调用到callback；1代表还要继续读；-1代表链接断开
-            if self._try_inline_read() <= 0:
-                break
+            ret, data = self._try_inline_read()
+            if ret <= 0:
+                return data
 
     @count_reader
-    def read_until(self, delimiter, callback):
+    def read_until(self, delimiter):
         """
         为了兼容调用的方法
         """
 
-        self._set_read_callback(callback)
         self._read_delimiter = delimiter
         while True:
-            # 0 代表调用到callback；1代表还要继续读；-1代表链接断开
-            if self._try_inline_read() <= 0:
-                break
+            ret, data = self._try_inline_read()
+            if ret <= 0:
+                return data
 
     @count_reader
-    def read_bytes(self, num_bytes, callback, streaming_callback=None):
-        """Run callback when we read the given number of bytes.
-
-        If a ``streaming_callback`` is given, it will be called with chunks
-        of data as they become available, and the argument to the final
-        ``callback`` will be empty.  Otherwise, the ``callback`` gets
-        the data as an argument.
+    def read_bytes(self, num_bytes):
+        """Run when we read the given number of bytes.
         """
-        self._set_read_callback(callback)
         assert isinstance(num_bytes, numbers.Integral)
         self._read_bytes = num_bytes
-        self._streaming_callback = streaming_callback
         while True:
-            # 0 代表调用到callback；1代表还要继续读；-1代表链接断开
-            if self._try_inline_read() <= 0:
-                break
+            ret, data = self._try_inline_read()
+            if ret <= 0:
+                return data
 
     @count_reader
-    def read_until_close(self, callback, streaming_callback=None):
+    def read_until_close(self):
         """Reads all data from the socket until it is closed.
-
-        If a ``streaming_callback`` is given, it will be called with chunks
-        of data as they become available, and the argument to the final
-        ``callback`` will be empty.  Otherwise, the ``callback`` gets the
-        data as an argument.
-
-        Subject to ``max_buffer_size`` limit from `IOStream` constructor if
-        a ``streaming_callback`` is not used.
         """
-        self._set_read_callback(callback)
-        self._streaming_callback = streaming_callback
         if self.closed():
-            if self._streaming_callback is not None:
-                self._run_callback(self._streaming_callback,
-                                   self._consume(self._read_buffer_size))
-            self._run_callback(self._read_callback,
-                               self._consume(self._read_buffer_size))
-            self._streaming_callback = None
-            self._read_callback = None
-            return
+            return self._consume(self._read_buffer_size)
 
         self._read_until_close = True
-        self._streaming_callback = streaming_callback
         while True:
-            # 0 代表调用到callback；1代表还要继续读；-1代表链接断开
-            if self._try_inline_read() < 0:
-                break
+            ret, data = self._try_inline_read()
+            if ret <= 0:
+                return data
 
     @count_reader
-    def read_with_checker(self, checker, callback):
+    def read_with_checker(self, checker):
         """
         checker(buf):
             0 继续接收
@@ -190,32 +149,29 @@ class Stream(object):
             <0 异常
         """
 
-        self._set_read_callback(callback)
         self._read_checker = checker
         while True:
-            # 0 代表调用到callback；1代表还要继续读；-1代表链接断开
-            if self._try_inline_read() <= 0:
-                break
+            ret, data = self._try_inline_read()
+            if ret <= 0:
+                return data
 
     @count_writer
-    def write(self, data, callback=None):
+    def write(self, data):
         """
         写数据
         """
 
         if self.closed():
-            return
+            return -1
 
         while data:
             num_bytes = safe_call(self.write_to_fd, data)
             if num_bytes is None:
-                logger.error('write num_bytes: %s', num_bytes)
-                break
+                return -2
 
             data = data[num_bytes:]
 
-        if callback:
-            self._run_callback(callback)
+        return 0
 
     def closed(self):
         return not self.sock
@@ -236,59 +192,44 @@ class Stream(object):
                       socket.IPPROTO_TCP, socket.TCP_NODELAY, 1 if value else 0)
 
     def read_from_fd(self):
-        return self.sock.recv(self.read_chunk_size)
+        chunk = self.sock.recv(self.read_chunk_size)
+
+        if not chunk:
+            self.close()
+            return None
+        return chunk
 
     def write_to_fd(self, data):
         return self.sock.send(data)
 
-    def _run_callback(self, callback, *args):
-        """
-        替换为kola的safe_call
-        """
-        return safe_call(callback, *args)
-
-    def _set_read_callback(self, callback):
-        assert not self._read_callback, "Already reading"
-        self._read_callback = callback
-
     def _try_inline_read(self):
         """Attempt to complete the current read operation from buffered data.
 
-        If the read can be completed without blocking, schedules the
-        read callback on the next IOLoop iteration; otherwise starts
-        listening for reads on the socket.
+        0 代表获取到数据了；1代表还要继续读；-1代表链接断开
         """
         # See if we've already got the data from a previous read
-        if self._read_from_buffer():
-            return 0
+        data = self._read_from_buffer()
+        if data:
+            return 0, data
 
         if self._read_to_buffer() == 0:
             # 说明断连接了
             self.close()
 
             if self._read_until_close:
-                if (self._streaming_callback is not None and
-                        self._read_buffer_size):
-                    self._run_callback(self._streaming_callback,
-                                       self._consume(self._read_buffer_size))
-                callback = self._read_callback
-                self._read_callback = None
                 self._read_until_close = False
-                self._run_callback(callback,
-                                   self._consume(self._read_buffer_size))
 
-            if self._close_callback:
-                self._run_callback(self._close_callback)
-                self._close_callback = None
+                return -1, self._consume(self._read_buffer_size)
 
             # 直接返回，因为buffer里面一定没数据了
-            return -1
+            return -1, None
 
-        if self._read_from_buffer():
+        data = self._read_from_buffer()
+        if data:
             # 收到了新的数据，判断下是否满足要求
-            return 0
+            return 0, data
 
-        return 1
+        return 1, None
 
     def _read_to_buffer(self):
         """Reads from the socket and appends the result to the read buffer.
@@ -313,23 +254,12 @@ class Stream(object):
     def _read_from_buffer(self):
         """Attempts to complete the currently-pending read from the buffer.
 
-        Returns True if the read was completed.
+        Returns data if the read was completed.
         """
-        if self._streaming_callback is not None and self._read_buffer_size:
-            bytes_to_consume = self._read_buffer_size
-            if self._read_bytes is not None:
-                bytes_to_consume = min(self._read_bytes, bytes_to_consume)
-                self._read_bytes -= bytes_to_consume
-            self._run_callback(self._streaming_callback,
-                               self._consume(bytes_to_consume))
         if self._read_bytes is not None and self._read_buffer_size >= self._read_bytes:
             num_bytes = self._read_bytes
-            callback = self._read_callback
-            self._read_callback = None
-            self._streaming_callback = None
             self._read_bytes = None
-            self._run_callback(callback, self._consume(num_bytes))
-            return True
+            return self._consume(num_bytes)
         elif self._read_delimiter is not None:
             # Multi-byte delimiters (e.g. '\r\n') may straddle two
             # chunks in the read buffer, so we can't easily find them
@@ -343,14 +273,9 @@ class Stream(object):
                 while True:
                     loc = self._read_buffer[0].find(self._read_delimiter)
                     if loc != -1:
-                        callback = self._read_callback
                         delimiter_len = len(self._read_delimiter)
-                        self._read_callback = None
-                        self._streaming_callback = None
                         self._read_delimiter = None
-                        self._run_callback(callback,
-                                           self._consume(loc + delimiter_len))
-                        return True
+                        return self._consume(loc + delimiter_len)
                     if len(self._read_buffer) == 1:
                         break
                     _double_prefix(self._read_buffer)
@@ -359,12 +284,8 @@ class Stream(object):
                 while True:
                     m = self._read_regex.search(self._read_buffer[0])
                     if m is not None:
-                        callback = self._read_callback
-                        self._read_callback = None
-                        self._streaming_callback = None
                         self._read_regex = None
-                        self._run_callback(callback, self._consume(m.end()))
-                        return True
+                        return self._consume(m.end())
                     if len(self._read_buffer) == 1:
                         break
                     _double_prefix(self._read_buffer)
@@ -374,13 +295,8 @@ class Stream(object):
                     loc, obj = self._read_checker(self._read_buffer[0])
                     if loc > 0:
                         # 说明就是要这些长度
-                        callback = self._read_callback
-                        self._read_callback = None
-                        self._streaming_callback = None
-                        self._read_delimiter = None
-                        self._run_callback(callback,
-                                           self._consume(loc))
-                        return True
+                        self._read_checker = None
+                        return self._consume(loc)
                     elif loc < 0:
                         # 说明接受的数据已经有问题了，直接把数据删掉，并退出
                         self._read_buffer.popleft()
@@ -389,7 +305,7 @@ class Stream(object):
                     if len(self._read_buffer) == 1:
                         break
                     _double_prefix(self._read_buffer)
-        return False
+        return None
 
     def _consume(self, loc):
         if loc == 0:
